###############################################################################
# Copyright (c) 2017, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory
# Written by Francesco Di Natale, dinatale3@llnl.gov.
#
# LLNL-CODE-734340
# All rights reserved.
# This file is part of MaestroWF, Version: 1.0.0.
#
# For details, see https://github.com/LLNL/maestrowf.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
###############################################################################

"""Class representing the sourcing of a script."""

import logging
import re

from maestrowf.abstracts import Source

logger = logging.getLogger(__name__)


class Script(Source):
    """Script class for applying changes to the execution environment."""

    # TODO: Sourcing is an issue. We need to figure out a way to handle this.
    def __init__(self, source):
        """
        Initialize the Script class.

        :params source: The command for changing the execution environment.
        """
        self.source = source
        self._verification("Script initialized without complete settings. Set"
                           " source before calling methods.")

    def apply(self, cmds):
        """
        Apply the Script source to the specified list of commands.

        :param cmds: List of commands to add source to.
        :returns: List of commands with the source prepended.
        """
        return [self.source] + list(cmds)

    def _verify(self):
        """
        Verify the Script object's contents.

        :returns: True if the Script object is valid, False otherwise.
        """
        valid_param_pattern = re.compile(r"\w+")
        return bool(re.search(valid_param_pattern, self.source))



from os import path
from maestrowf import __version__
from setuptools import setup, find_packages


def load_readme():
    """
    Load the readme from the root of the package directory.

    :returns: A string containing the contents of README.md.
    """
    pkg_path = path.abspath(path.dirname(__file__))
    with open(path.join(pkg_path, 'README.md')) as f:
        long_description = f.read()

    return long_description


setup(
    name='maestrowf',
    description='A tool to easily orchestrate general computational workflows '
    'both locally and on supercomputers.',
    version=__version__,
    author='Francesco Di Natale',
    maintainer='Francesco Di Natale',
    author_email='dinatale3@llnl.gov',
    url='https://github.com/llnl/maestrowf',
    license='MIT License',
    packages=find_packages(),
    entry_points={
        'console_scripts': [
                'maestro = maestrowf.maestro:main',
                'conductor = maestrowf.conductor:main',
        ]
    },
    install_requires=[
        'PyYAML>=4.2b1',
        'six',
        "filelock",
        "tabulate",
        "enum34 ; python_version<'3.4'",
        "dill",
        "jsonschema>=3.2.0",
        "coloredlogs",
        "chainmap ; python_version<'3'",
    ],
    extras_require={},
    long_description=load_readme(),
    long_description_content_type='text/markdown',
    download_url='https://pypi.org/project/maestrowf/',
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Operating System :: Unix',
        'Operating System :: MacOS :: MacOS X',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering',
        'Topic :: System :: Distributed Computing',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    package_data={
        'maestrowf': [
            'maestrowf/specification/schemas/yamlspecification.json'
        ],
    },
    include_package_data=True,
)
